#!/usr/bin/env python
""" 
 * synnova.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This sw_upload is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this sw_upload are reserved by
 *  Synerty Pty Ltd
 *
"""
from pytmpdir.Directory import DirSettings
from txhttputil.site.FileUploadRequest import FileUploadRequest
from txhttputil.util.LoggingUtil import setupLogging

from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_server import importPackages
from peek_server.storage import setupDbConn
from peek_server.storage.DeclarativeBase import metadata
from vortex.VortexFactory import VortexFactory

setupLogging()

import logging
import os

from twisted.internet import reactor, defer

from txhttputil.site.SiteUtil import setupSite

logger = logging.getLogger(__name__)

# ------------------------------------------------------------------------------
# Set the parallelism of the database and reactor

reactor.suggestThreadPoolSize(10)
defer.setDebugging(True)



def setupPlatform():
    from peek_platform import PeekPlatformConfig
    PeekPlatformConfig.componentName = peekServerName

    # Tell the platform classes about our instance of the pluginSwInstallManager
    from peek_server.server.sw_install.PluginSwInstallManager import PluginSwInstallManager
    PeekPlatformConfig.pluginSwInstallManager = PluginSwInstallManager()

    # Tell the platform classes about our instance of the PeekSwInstallManager
    from peek_server.server.sw_install.PeekSwInstallManager import PeekSwInstallManager
    PeekPlatformConfig.peekSwInstallManager = PeekSwInstallManager()

    # Tell the platform classes about our instance of the PeekLoaderBase
    from peek_server.plugin.ServerPluginLoader import ServerPluginLoader
    PeekPlatformConfig.pluginLoader = ServerPluginLoader()

    # The config depends on the componentName, order is important
    from peek_server.PeekServerConfig import PeekServerConfig
    PeekPlatformConfig.config = PeekServerConfig()

    # Set default logging level
    logging.root.setLevel(PeekPlatformConfig.config.loggingLevel)

    # Set paths for the Directory object
    DirSettings.defaultDirChmod = PeekPlatformConfig.config.DEFAULT_DIR_CHMOD
    DirSettings.tmpDirPath = PeekPlatformConfig.config.tmpPath
    FileUploadRequest.tmpFilePath = PeekPlatformConfig.config.tmpPath


def main():
    # defer.setDebugging(True)
    # sys.argv.remove(DEBUG_ARG)
    # import pydevd
    # pydevd.settrace(suspend=False)

    setupPlatform()
    from peek_platform import PeekPlatformConfig
    import peek_server

    # Configure sqlalchemy
    setupDbConn(
        metadata=metadata,
        dbEngineArgs = PeekPlatformConfig.config.dbEngineArgs,
        dbConnectString=PeekPlatformConfig.config.dbConnectString,
        alembicDir=os.path.join(os.path.dirname(peek_server.__file__), "alembic")
    )

    # Force model migration
    from peek_server.storage import dbConn
    dbConn.migrate()

    # Import remaining components
    importPackages()

    # Load all plugins
    PeekPlatformConfig.pluginLoader.loadAllPlugins()

    from peek_server.backend.SiteRootResource import setup as setupSiteRoot
    from peek_server.backend.SiteRootResource import root as siteRoot
    setupSiteRoot()

    setupSite("Peek Admin",
              siteRoot,
              PeekPlatformConfig.config.sitePort,
              enableLogin=False)

    from peek_server.server.PeekServerPlatformRootResource import setup as setupPlatRoot
    from peek_server.server.PeekServerPlatformRootResource import root as platformRoot
    setupPlatRoot()

    setupSite("Peek Platform Data Exchange",
              platformRoot,
              PeekPlatformConfig.config.peekServerHttpPort,
              enableLogin=False)

    VortexFactory.createTcpServer(name=PeekPlatformConfig.componentName,
                                  port=PeekPlatformConfig.config.peekServerVortexTcpPort)


    webSocketPort = PeekPlatformConfig.config.webSocketPort
    VortexFactory.createWebsocketServer(PeekPlatformConfig.componentName, webSocketPort)

    reactor.addSystemEventTrigger('before', 'shutdown',
                                  PeekPlatformConfig.pluginLoader.unloadAllPlugins)

    reactor.run()


if __name__ == '__main__':
    main()



""" 
 * view.common.uiobj.Style.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import json
import logging

from twisted.web.server import NOT_DONE_YET
from txhttputil.site.BasicResource import BasicResource

from peek_platform import PeekPlatformConfig
from peek_server.server.sw_upload.PluginSwUploadManager import PluginSwUploadManager
from peek_server.server.sw_upload.PeekSwUploadManager import PeekSwUploadManager

logger = logging.getLogger(name=__name__)


class PeekSwUploadResource(BasicResource):
    isLeaf = True
    useLargeRequest = True

    UPDATE_TYPE_PLATFORM = 0
    UPDATE_TYPE_PLUGIN = 1

    def __init__(self, updateType, *args):
        BasicResource.__init__(self, *args)

        self._updateType = updateType
        self._desc = {self.UPDATE_TYPE_PLATFORM: "Peek Platform",
                      self.UPDATE_TYPE_PLUGIN: "Peek App",
                      }[self._updateType]

    def render_GET(self, request):
        # TODO, This is where the platform gets it's lates download from
        raise Exception("Updates must be done via the UI")

    def render_POST(self, request):
        request.responseHeaders.setRawHeaders('content-type', ['text/plain'])
        logger.info("received %s sw_upload update request" % self._desc)

        try:
            if PeekPlatformConfig.config.capabilities['supportExceeded']:
                return json.dumps({'error': "License has expired, Updates not allowed"})

        except Exception as e:
            return json.dumps({'error': str(e.message)})

        updateManager = {self.UPDATE_TYPE_PLATFORM: PeekSwUploadManager,
                         self.UPDATE_TYPE_PLUGIN: PluginSwUploadManager,
                         }[self._updateType]()

        def good(data):
            request.write(json.dumps({'message': str(data)}).encode())
            request.finish()
            logger.info("Finished updating %s sw_upload" % self._desc)

        def bad(failure):
            e = failure.value
            logger.exception(e)

            request.write(json.dumps(
                {'error': str(failure.value),
                 'stdout': e.stdout if hasattr(e, 'stdout') else None,
                 'stderr': e.stderr if hasattr(e, 'stderr') else None}).encode())

            request.finish()

        d = updateManager.processUpdate(request.content.namedTemporaryFile)
        d.addCallbacks(good, bad)

        def closedError(failure):
            logger.error("Got closedError %s" % failure)

        request.notifyFinish().addErrback(closedError)

        return NOT_DONE_YET



import logging
import os

from typing import Type, Tuple

# from peek_platform.plugin.PluginFrontendInstallerABC import PluginFrontendInstallerABC
from peek_platform.frontend.WebBuilder import WebBuilder
from peek_platform.plugin.PluginLoaderABC import PluginLoaderABC
from peek_plugin_base.PluginCommonEntryHookABC import PluginCommonEntryHookABC
from peek_plugin_base.server.PluginServerEntryHookABC import PluginServerEntryHookABC
from peek_plugin_base.server.PluginServerStorageEntryHookABC import \
    PluginServerStorageEntryHookABC
from peek_plugin_base.server.PluginServerWorkerEntryHookABC import \
    PluginServerWorkerEntryHookABC
from peek_server.plugin.PeekServerPlatformHook import PeekServerPlatformHook

logger = logging.getLogger(__name__)


class ServerPluginLoader(PluginLoaderABC):
    _instance = None

    def __new__(cls, *args, **kwargs):
        assert cls._instance is None, "ServerPluginLoader is a singleton, don't construct it"
        cls._instance = PluginLoaderABC.__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self, *args, **kwargs):
        PluginLoaderABC.__init__(self, *args, **kwargs)

    @property
    def _entryHookFuncName(self) -> str:
        return "peekServerEntryHook"

    @property
    def _entryHookClassType(self):
        return PluginServerEntryHookABC

    @property
    def _platformServiceNames(self) -> [str]:
        return ["server", "storage"]

    def loadAllPlugins(self):
        PluginLoaderABC.loadAllPlugins(self)

        import peek_admin
        frontendProjectDir = os.path.dirname(peek_admin.__file__)

        from peek_platform import PeekPlatformConfig
        PeekPlatformConfig.config

        webBuilder = WebBuilder(frontendProjectDir,
                                "peek-admin",
                                PeekPlatformConfig.config,
                                self._loadedPlugins.values())
        webBuilder.build()

    def unloadPlugin(self, pluginName: str):
        PluginLoaderABC.unloadPlugin(self, pluginName)

        # Remove the Plugin resource tree

        from peek_server.backend.SiteRootResource import root as serverRootResource
        try:
            serverRootResource.deleteChild(pluginName.encode())
        except KeyError:
            pass

    def _loadPluginThrows(self, pluginName: str,
                          EntryHookClass: Type[PluginCommonEntryHookABC],
                          pluginRootDir: str,
                          requiresService: Tuple[str, ...]) -> None:
        # Everyone gets their own instance of the plugin API
        platformApi = PeekServerPlatformHook()

        pluginMain = EntryHookClass(pluginName=pluginName,
                                    pluginRootDir=pluginRootDir,
                                    platform=platformApi)

        # Load the plugin
        pluginMain.load()

        if isinstance(pluginMain, PluginServerWorkerEntryHookABC):
            # Configure the celery app in the worker
            # This is not the worker that will be started, it allows the worker to queue tasks
            from peek_platform.CeleryApp import configureCeleryApp
            configureCeleryApp(pluginMain.celeryApp)

        # Check the implementation
        elif "worker" in requiresService:
            raise Exception("Plugin %s requires 'worker' service."
                            " It must now inherit PluginServerWorkerEntryHookABC"
                            " in its PluginServerEntryHook implementation")

        if isinstance(pluginMain, PluginServerStorageEntryHookABC):

            metadata = pluginMain.dbMetadata
            schemaName = pluginName.replace("peek_plugin_", "pl_")
            if metadata.schema != schemaName:
                raise Exception("Peek plugin %s db schema name is %s, should be %s"
                                % (pluginName, metadata.schema, schemaName))

            # Create/Migrate the database schema
            pluginMain._migrateStorageSchema(pluginMain.dbMetadata)

        # Check the implementation
        elif "storage" in requiresService:
            raise Exception("Plugin %s requires 'storage' service."
                            " It must now inherit PluginServerStorageEntryHookMixin"
                            " in its PluginServerEntryHook implementation")


        # Start the Plugin
        pluginMain.start()

        # Add all the resources required to serve the backend site
        # And all the plugin custom resources it may create

        from peek_server.backend.SiteRootResource import root
        root.putChild(pluginName.encode(), platformApi.rootResource)

        self._loadedPlugins[pluginName] = pluginMain



"""Added Peek Environent tables

Revision ID: 7923ec94177a
Revises: bc1622f6c16c
Create Date: 2016-12-03 16:58:10.013775

"""

# revision identifiers, used by Alembic.
revision = '7923ec94177a'
down_revision = 'bc1622f6c16c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('PeekEnvServer',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(50), nullable=False),
    sa.Column('description', sa.String(), nullable=True),
    sa.Column('ip', sa.String(40), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('ip'),
    sa.UniqueConstraint('name'),
    schema='peek_server'
    )
    op.create_table('PeekEnvAgent',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(50), nullable=False),
    sa.Column('description', sa.String(), nullable=True),
    sa.Column('ip', sa.String(40), nullable=True),
    sa.Column('serverId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['serverId'], ['peek_server.PeekEnvServer.id'], ),
    sa.PrimaryKeyConstraint('id', 'serverId'),
    sa.UniqueConstraint('ip'),
    sa.UniqueConstraint('name'),
    schema='peek_server'
    )
    op.create_table('PeekEnvClient',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(50), nullable=False),
    sa.Column('description', sa.String(), nullable=True),
    sa.Column('ip', sa.String(40), nullable=True),
    sa.Column('serverId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['serverId'], ['peek_server.PeekEnvServer.id'], ),
    sa.PrimaryKeyConstraint('id', 'serverId'),
    sa.UniqueConstraint('ip'),
    sa.UniqueConstraint('name'),
    schema='peek_server'
    )
    op.create_table('PeekEnvWorker',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(50), nullable=False),
    sa.Column('description', sa.String(), nullable=True),
    sa.Column('ip', sa.String(40), nullable=True),
    sa.Column('serverId', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['serverId'], ['peek_server.PeekEnvServer.id'], ),
    sa.PrimaryKeyConstraint('id', 'serverId'),
    sa.UniqueConstraint('ip'),
    sa.UniqueConstraint('name'),
    schema='peek_server'
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('PeekEnvWorker', schema='peek_server')
    op.drop_table('PeekEnvClient', schema='peek_server')
    op.drop_table('PeekEnvAgent', schema='peek_server')
    op.drop_table('PeekEnvServer', schema='peek_server')
    ### end Alembic commands ###



'''
Created on 09/07/2014

@author: synerty
'''

from vortex.Tuple import addTupleType, Tuple, TupleField


@addTupleType
class PeekAdmNavbarUserTuple(Tuple):
    __tupleType__ = 'peek_server.PeekAdmNavbarUserTuple'

    supportExceeded = TupleField(False)
    demoExceeded = TupleField(False)
    countsExceeded = TupleField(False)
    username = TupleField('nouser')


'''
Created on 09/07/2014

@author: synerty
'''
from peek_platform import PeekPlatformConfig
from peek_server.backend.navbar.PeekAdmNavbarHandler import navbarDataHandler
from vortex.DataWrapTuple import DataWrapTuple
from vortex.Payload import Payload
from vortex.PayloadEndpoint import PayloadEndpoint
from vortex.VortexABC import SendVortexMsgResponseCallable

capabilitiesDataFiltKey = {
    "plugin": "peek_server",
    "key": "admin.capabilities.data"
}


# HANDLER
class __CrudHandler():
    def __init__(self, payloadFilter):
        self._payloadFilter = payloadFilter
        self._ep = PayloadEndpoint(self._payloadFilter, self.process)

    def process(self, payload,
                vortexUuid,
                httpSession,
                sendResponse: SendVortexMsgResponseCallable,
                              **kwargs):
        # from peek_server.server.auth import AuthValue
        from peek_server.storage.Setting import internalSetting, CAPABILITIES_KEY

        result = None
        from peek_server.storage import dbConn
        session = dbConn.ormSessionCreator

        # Force capabilities reload on page load
        PeekPlatformConfig.config._capabilities = None

        # This is an update
        if payload.tuples:
            dataWrapTuple = payload.tuples[0]
            self._capabilities = None

            # try:
            #     AuthValue.loadCapabilities(self, dataWrapTuple.data)
            # except Exception as e:
            #     pass
            if self._capabilities is None:
                result = {"success": False,
                          "message": "The license entered is not valid for this server"}
                sendResponse(Payload(filt=self._payloadFilter,
                                          result=result).toVortexMsg())
                return

            PeekPlatformConfig.config._capabilities = self._capabilities

            internalSetting()[CAPABILITIES_KEY] = dataWrapTuple.data
            session.commit()
            result = {"success": True,
                      "message": "The license was successfully loaded"}
        dataWrapTuple = DataWrapTuple()
        dataWrapTuple.data = PeekPlatformConfig.config.capabilities

        sendResponse(Payload(filt=self._payloadFilter,
                                  tuples=[dataWrapTuple],
                                  result=result).toVortexMsg())

        navbarDataHandler.sendModelUpdate(vortexUuid=vortexUuid)


__ormCrudHandler = __CrudHandler(capabilitiesDataFiltKey)



'''
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
 * Website : http://www.synerty.com
 * Support : support@synerty.com
 *
'''
import logging
import os

from jsoncfg.value_mappers import require_string, require_integer

from peek_platform.file_config.PeekFileConfigABC import PeekFileConfigABC
from peek_platform.file_config.PeekFileConfigFrontendDirMixin import \
    PeekFileConfigFrontendDirMixin
from peek_platform.file_config.PeekFileConfigOsMixin import PeekFileConfigOsMixin
from peek_platform.file_config.PeekFileConfigPlatformMixin import PeekFileConfigPlatformMixin
from peek_platform.file_config.PeekFileConfigSqlAlchemyMixin import \
    PeekFileConfigSqlAlchemyMixin
from peek_server.PeekServerConfigLicMixin import PeekServerConfigLicMixin

logger = logging.getLogger(__name__)


class PeekServerConfig(PeekFileConfigABC,
                       PeekFileConfigOsMixin,
                       PeekFileConfigPlatformMixin,
                       PeekFileConfigSqlAlchemyMixin,
                       PeekServerConfigLicMixin,
                       PeekFileConfigFrontendDirMixin):
    """
    This class creates a server configuration
    """

    import peek_admin
    _frontendProjectDir = os.path.dirname(peek_admin.__file__)

    ### USER SECTION ###
    @property
    def adminPass(self):
        with self._cfg as c:
            return c.user.admin["pass"]("peeking", require_string)

    @property
    def adminUser(self):
        with self._cfg as c:
            return c.user.admin.user("backend", require_string)

    ### SERVER SECTION ###
    @property
    def sitePort(self) -> int:
        """ Site Port

        The port used to serve the admin web page on
        """
        with self._cfg as c:
            return c.server.sitePort(8010, require_integer)

    @property
    def webSocketPort(self) -> int:
        with self._cfg as c:
            return c.server.webSocketPort(8013, require_integer)


    ### SERVER SECTION ###
    @property
    def peekServerHttpPort(self):
        with self._cfg as c:
            return c.peekServer.httpPort(8011, require_integer)

    @property
    def peekServerVortexTcpPort(self):
        with self._cfg as c:
            return c.peekServer.tcpVortexPort(8012, require_integer)


"""Initial setup

Revision ID: 05cf60a0fd22
Revises: 7923ec94177a
Create Date: 2017-01-16 13:48:40.525749

"""

# revision identifiers, used by Alembic.
revision = '05cf60a0fd22'
down_revision = '7923ec94177a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



""" 
 * orm.ExtensionData.py
 *
 *  Copyright Synerty Pty Ltd 2011
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import logging

from sqlalchemy import Column
from sqlalchemy import Integer, String
from sqlalchemy.orm import relationship
from sqlalchemy.sql.schema import ForeignKey

from peek_server.storage.DeclarativeBase import DeclarativeBase
from vortex.Tuple import Tuple, addTupleType

logger = logging.getLogger(__name__)


@addTupleType
class PeekEnvServer(Tuple, DeclarativeBase):
    """ PeekPluginInfo

    This table stores information on the version of Peek apps that are stored in Peek.

    """
    __tupleType__ = 'peek_server.env.server'
    __tablename__ = 'PeekEnvServer'

    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False, unique=True)
    description = Column(String, nullable=True)
    ip = Column(String(40), nullable=True, unique=True)

    workers = relationship("PeekEnvWorker")
    agent = relationship("PeekEnvAgent")


@addTupleType
class PeekEnvWorker(Tuple, DeclarativeBase):
    """ PeekPluginInfo

    This table stores information on the version of Peek apps that are stored in Peek.

    """
    __tupleType__ = 'peek_server.env.worker'
    __tablename__ = 'PeekEnvWorker'

    id = Column(Integer, primary_key=True)
    name = Column(String(50), nullable=False, unique=True)
    description = Column(String, nullable=True)
    ip = Column(String(40), nullable=True, unique=True)

    serverId = Column(ForeignKey('peek_server.PeekEnvServer.id'), primary_key=True)
    server = relationship("PeekEnvServer")


@addTupleType
class PeekEnvAgent(Tuple, DeclarativeBase):
    """ PeekPluginInfo

    This table stores information on the version of Peek apps that are stored in Peek.

    """
    __tupleType__ = 'peek_server.env.agent'
    __tablename__ = 'PeekEnvAgent'

    id = Column(Integer, primary_key=True)
    name = Column(String(50), nullable=False, unique=True)
    description = Column(String, nullable=True)
    ip = Column(String(40), nullable=True, unique=True)

    serverId = Column(ForeignKey('peek_server.PeekEnvServer.id'), primary_key=True)
    server = relationship("PeekEnvServer")


@addTupleType
class PeekEnvClient(Tuple, DeclarativeBase):
    """ PeekPluginInfo

    This table stores information on the version of Peek apps that are stored in Peek.

    """
    __tupleType__ = 'peek_server.env.client'
    __tablename__ = 'PeekEnvClient'

    id = Column(Integer, primary_key=True)
    name = Column(String(50), nullable=False, unique=True)
    description = Column(String, nullable=True)
    ip = Column(String(40), nullable=True, unique=True)

    serverId = Column(ForeignKey('peek_server.PeekEnvServer.id'), primary_key=True)
    server = relationship("PeekEnvServer")



'''
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
 * Website : http://www.synerty.com
 * Support : support@synerty.com
 *
'''
import logging
from abc import ABCMeta
from datetime import datetime

from peek_platform.file_config.PeekFileConfigABC import PeekFileConfigABC

logger = logging.getLogger(__name__)


class PeekServerConfigLicMixin:
    ### CAPABILITIES SECTION ###
    @property
    def capabilities(self):

        # from peek_server.server.auth import AuthValue
        from peek_server.storage.Setting import globalSetting, SYSTEM_NAME
        from peek_server.storage.Setting import internalSetting, CAPABILITIES_KEY

        if not hasattr(self, '_capabilities'):
            self._capabilities = None

        # if not self._capabilities:
        #     data = internalSetting()[CAPABILITIES_KEY]
        #     AuthValue.loadCapabilities(self, data)

        if not self._capabilities:
            self._capabilities = {'sid': 'SER-VER-ID-01', #AuthValue.authKey(),
                                  'upd': "01-Jan-2014",

                                  'lil': 20,
                                  'lin': "Unlicensed",

                                  'sun': "Not Supported",
                                  'sul': 0,
                                  'aud': False,
                                  'dii': False,

                                  }

        self._capabilities['upd'] = "01-Jan-2018"  # DEBUG

        lic = self._capabilities
        # session = getNovaOrmSession()

        # def limitText(countKey, limitKey):
        #     limit = lic[limitKey]
        #     count = lic[countKey]
        #     return "%s out of %s" % (count, ("Unlimited" if limit == -1 else limit))

        lic['serverName'] = globalSetting()[SYSTEM_NAME]

        # lic['planCount'] = session.query(Plan).count()
        # lic['planCountText'] = limitText('planCount', 'plc')
        # lic['planCountExceeded'] = (lic['plc'] != -1
        #                             and lic['planCount'] > lic['plc'])
        #
        #
        # lic['execHistoryCountText'] = ("Unlimited" if lic['ehc'] == -1 else lic['ehc'])

        upd = datetime.strptime(lic['upd'], '%d-%b-%Y')
        lic['supportExceeded'] = upd < datetime.today()
        lic['demoExceeded'] = lic['lil'] == 20 and lic['supportExceeded']
        lic['countsExceeded'] = False
        # (lic['planCountExceeded']
        #                      or lic['scheduleCountExceeded']
        #                      or lic['placevalueCountExceeded'])

        if lic['supportExceeded']:
            lic['sun'] = "Expired"

        return self._capabilities



"""Added initial tables

Revision ID: c5e78990adf6
Revises: b7dda7cf354c
Create Date: 2016-11-03 19:08:58.489098

"""

# revision identifiers, used by Alembic.
revision = 'c5e78990adf6'
down_revision = '3aa82a9abe67'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('PeekPluginInfo',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('title', sa.String(), nullable=False),
    sa.Column('name', sa.String(50), nullable=False),
    sa.Column('version', sa.String(40), nullable=False),
    sa.Column('creator', sa.String(), nullable=True),
    sa.Column('website', sa.String(), nullable=True),
    sa.Column('buildNumber', sa.String(), nullable=True),
    sa.Column('buildDate', sa.String(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    schema='peek_server'
    )
    op.create_index('idx_PeekPluginInfo_NameVersion', 'PeekPluginInfo', ['name', 'version'], unique=True, schema='peek_server')
    op.create_table('Setting',
    sa.Column('id', sa.Integer(), nullable=False, autoincrement=True),
    sa.Column('name', sa.String(50), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    schema='peek_server'
    )
    op.create_table('SettingProperty',
    sa.Column('id', sa.Integer(), nullable=False, autoincrement=True),
    sa.Column('settingId', sa.Integer(), nullable=False),
    sa.Column('key', sa.String(60), nullable=False),
    sa.Column('type', sa.String(length=16), nullable=True),
    sa.Column('int_value', sa.Integer(), nullable=True),
    sa.Column('char_value', sa.String(), nullable=True),
    sa.Column('boolean_value', sa.Boolean(), nullable=True),
    sa.ForeignKeyConstraint(['settingId'], ['peek_server.Setting.id'], ),
    sa.PrimaryKeyConstraint('id', 'settingId', 'key'),
    schema='peek_server'
    )
    op.create_index('idx_SettingProperty_settingId', 'SettingProperty', ['settingId'], unique=False, schema='peek_server')
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('idx_SettingProperty_settingId', table_name='SettingProperty', schema='peek_server')
    op.drop_table('SettingProperty', schema='peek_server')
    op.drop_table('Setting', schema='peek_server')
    op.drop_index('idx_PeekPluginInfo_NameVersion', table_name='PeekPluginInfo', schema='peek_server')
    op.drop_table('PeekPluginInfo', schema='peek_server')
    ### end Alembic commands ###



"""enabled postgis

Revision ID: 3aa82a9abe67
Revises: 5ad8b58df646
Create Date: 2015-06-02 22:09:51.824555

"""

# revision identifiers, used by Alembic.
revision = '3aa82a9abe67'
down_revision = '5ad8b58df646'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():

  sql = '''
          -- Enable PostGIS (includes raster)
          CREATE EXTENSION postgis;
          -- Enable Topology
          CREATE EXTENSION postgis_topology;
          -- fuzzy matching needed for Tiger
          CREATE EXTENSION fuzzystrmatch;
          -- Enable US Tiger Geocoder
          CREATE EXTENSION postgis_tiger_geocoder;
          -- commit
          COMMIT;
          '''
  # op.execute(sql)


def downgrade():
    pass



'''
Created on 09/07/2014

@author: synerty
'''
import txhttputil

from peek_server.storage import Setting, dbConn
from peek_server.storage.Setting import globalSetting
from vortex.sqla_orm.OrmCrudHandler import OrmCrudHandler

filtKey = {"plugin": "peek_server",
           "key": "server.setting.data"}


# HANDLER
class __CrudHandler(OrmCrudHandler):
    def createDeclarative(self, session, payloadFilt):
        return [p for p in globalSetting().propertyObjects]

    def postProcess(self, action, payloadFilt, vortexUuid):
        if action in [OrmCrudHandler.UPDATE, OrmCrudHandler.CREATE]:
            settings = globalSetting()
            txhttputil.DESCRIPTION = settings[Setting.SYSTEM_DESCRIPTION]
            txhttputil.TITLE = settings[Setting.SYSTEM_NAME]


__ormCrudHandler = __CrudHandler(lambda: dbConn.ormSessionCreator, Setting, filtKey)



import logging

from vortex.PayloadEndpoint import PayloadEndpoint
from vortex.VortexABC import SendVortexMsgResponseCallable

logger = logging.getLogger(__name__)

# -------------------------------------
# Payload echo for the platform
# Used for the platform to tell when the peek_server server restarts.

# The filter we listen on
peekServerEchoFilt = {
    'plugin' : 'peek_platform',
    'key': "peek_platform.echo"
}  # LISTEN / SEND


class PeekServerEchoHandler(object):
    def __init__(self, payloadFilter):
        self._ep = PayloadEndpoint(payloadFilter, self._process)

    def _process(self, payload,
                 sendResponse: SendVortexMsgResponseCallable,
                 **kwargs):
        sendResponse(payload.toVortexMsg())


__peekServerEchoHandler = PeekServerEchoHandler(peekServerEchoFilt)



from peek_platform import PeekPlatformConfig
from peek_plugin_base.PeekPlatformFrontendHookABC import PeekPlatformFrontendHookABC
from peek_plugin_base.server.PeekServerPlatformHookABC import PeekServerPlatformHookABC


class PeekServerPlatformHook(PeekServerPlatformHookABC):
    def __init__(self):
        PeekPlatformFrontendHookABC.__init__(self)

    @property
    def dbConnectString(self) -> str:
        from peek_platform import PeekPlatformConfig
        return PeekPlatformConfig.config.dbConnectString

    def _getOtherPluginApi(self, pluginName):
        serverPluginLoader = PeekPlatformConfig.pluginLoader

        otherPlugin = serverPluginLoader.pluginEntryHook(pluginName)
        if not otherPlugin:
            return None

        from peek_plugin_base.server.PluginServerEntryHookABC import \
            PluginServerEntryHookABC

        assert isinstance(otherPlugin, PluginServerEntryHookABC), (
            "Not an instance of PluginServerEntryHookABC")

        return otherPlugin

    def getOtherPluginApi(self, pluginName):
        otherPlugin = self._getOtherPluginApi(pluginName)
        return otherPlugin.publishedServerApi if otherPlugin else None

    def getOtherPluginStorageApi(self, pluginName):
        otherPlugin = self._getOtherPluginApi(pluginName)
        return otherPlugin.publishedStorageApi if otherPlugin else None
